@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,7 +23,8 @@
 # along with ldap3 in the COPYING and COPYING.LESSER files.
 # If not, see <http://www.gnu.org/licenses/>.
 
-from .. import RESULT_OPERATIONS_ERROR, RESULT_PROTOCOL_ERROR, RESULT_TIME_LIMIT_EXCEEDED, RESULT_SIZE_LIMIT_EXCEEDED, \
+from os import sep
+from .results import RESULT_OPERATIONS_ERROR, RESULT_PROTOCOL_ERROR, RESULT_TIME_LIMIT_EXCEEDED, RESULT_SIZE_LIMIT_EXCEEDED, \
     RESULT_STRONGER_AUTH_REQUIRED, RESULT_REFERRAL, RESULT_ADMIN_LIMIT_EXCEEDED, RESULT_UNAVAILABLE_CRITICAL_EXTENSION, \
     RESULT_AUTH_METHOD_NOT_SUPPORTED, RESULT_UNDEFINED_ATTRIBUTE_TYPE, RESULT_NO_SUCH_ATTRIBUTE, \
     RESULT_SASL_BIND_IN_PROGRESS, RESULT_CONFIDENTIALITY_REQUIRED, RESULT_INAPPROPRIATE_MATCHING, \
@@ -364,23 +365,15 @@ class LDAPKeyError(LDAPExceptionError, KeyError, AttributeError):
     pass
 
 
-class LDAPAttributeError(LDAPExceptionError, AttributeError):
-    pass
-
-
-class LDAPEntryError(LDAPExceptionError):
-    pass
-
-
 class LDAPObjectError(LDAPExceptionError, ValueError):
     pass
 
 
-class LDAPTypeError(LDAPExceptionError, TypeError):
+class LDAPAttributeError(LDAPExceptionError, ValueError, TypeError):
     pass
 
 
-class LDAPReaderError(LDAPExceptionError):
+class LDAPCursorError(LDAPExceptionError):
     pass
 
 
@@ -414,7 +407,7 @@ class LDAPConnectionIsReadOnlyError(LDAPExceptionError):
     pass
 
 
-class LDAPChangesError(LDAPExceptionError, ValueError):
+class LDAPChangeError(LDAPExceptionError, ValueError):
     pass
 
 
@@ -462,7 +455,11 @@ class LDAPInvalidDereferenceAliasesError(LDAPExceptionError, ValueError):
     pass
 
 
-class LDAPControlsError(LDAPExceptionError, ValueError):
+class LDAPInvalidValueError(LDAPExceptionError, ValueError):
+    pass
+
+
+class LDAPControlError(LDAPExceptionError, ValueError):
     pass
 
 
@@ -502,6 +499,10 @@ class LDAPResponseTimeoutError(LDAPExceptionError):
     pass
 
 
+class LDAPTransactionError(LDAPExceptionError):
+    pass
+
+
 # communication exceptions
 class LDAPCommunicationError(LDAPExceptionError):
     pass
@@ -551,18 +552,25 @@ class LDAPConnectionPoolNotStartedError(LDAPExceptionError):
 # restartable strategy
 class LDAPMaximumRetriesError(LDAPExceptionError):
     def __str__(self):
+        s = []
         if self.args:
             if isinstance(self.args, tuple):
                 if len(self.args) > 0:
-                    print('LDAPMaximumRetriesError: ' + self.args[0])
+                    s.append('LDAPMaximumRetriesError: ' + str(self.args[0]))
                 if len(self.args) > 1:
-                    print('Exception history:')
+                    s.append('Exception history:')
+                    prev_exc = ''
                     for i, exc in enumerate(self.args[1]):  # args[1] contains exception history
-                        print(str(i).rjust(5), str(exc[0]), ':', exc[1], '-', exc[2])
+                        if str(exc[1]) != prev_exc:
+                            s.append((str(i).rjust(5) + ' ' + str(exc[0]) + ': ' + str(exc[1]) + ' - ' + str(exc[2])))
+                            prev_exc = str(exc[1])
+
                 if len(self.args) > 2:
-                    print('Maximum number of retries reached: ' + str(self.args[2]))
+                    s.append('Maximum number of retries reached: ' + str(self.args[2]))
         else:
-            LDAPExceptionError.__str__(self)
+            s = [LDAPExceptionError.__str__(self)]
+
+        return sep.join(s)
 
 
 # exception factories
