@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -27,11 +27,10 @@ import socket
 from threading import Lock
 from datetime import datetime, MINYEAR
 
-from .. import NONE, DSA, SCHEMA, ALL, BASE, LDAP_MAX_INT, get_config_parameter, \
-    OFFLINE_EDIR_8_8_8, OFFLINE_AD_2012_R2, OFFLINE_SLAPD_2_4, OFFLINE_DS389_1_3_3, \
-    SEQUENCE_TYPES, IP_SYSTEM_DEFAULT, IP_V4_ONLY, IP_V6_ONLY, IP_V4_PREFERRED, IP_V6_PREFERRED, ADDRESS_INFO_REFRESH_TIME
+from .. import DSA, SCHEMA, ALL, BASE, get_config_parameter, OFFLINE_EDIR_8_8_8, OFFLINE_AD_2012_R2, OFFLINE_SLAPD_2_4, OFFLINE_DS389_1_3_3, SEQUENCE_TYPES, IP_SYSTEM_DEFAULT, IP_V4_ONLY, IP_V6_ONLY, IP_V4_PREFERRED, IP_V6_PREFERRED, STRING_TYPES
 from .exceptions import LDAPInvalidServerError, LDAPDefinitionError, LDAPInvalidPortError, LDAPInvalidTlsSpecificationError, LDAPSocketOpenError
 from ..protocol.formatters.standard import format_attribute_values
+from ..protocol.rfc4511 import LDAP_MAX_INT
 from ..protocol.rfc4512 import SchemaInfo, DsaInfo
 from .tls import Tls
 from ..utils.log import log, log_enabled, ERROR, BASIC, PROTOCOL
@@ -42,6 +41,7 @@ except ImportError:
     from urllib import unquote  # Python 2
 
 try:  # try to discover if unix sockets are available for LDAP over IPC (ldapi:// scheme)
+    # noinspection PyUnresolvedReferences
     from socket import AF_UNIX
     unix_socket_available = True
 except ImportError:
@@ -73,7 +73,7 @@ class Server(object):
                  port=None,
                  use_ssl=False,
                  allowed_referral_hosts=None,
-                 get_info=NONE,
+                 get_info=SCHEMA,
                  tls=None,
                  formatter=None,
                  connect_timeout=None,
@@ -81,6 +81,7 @@ class Server(object):
 
         self.ipc = False
         url_given = False
+        host = host.strip()
         if host.lower().startswith('ldap://'):
             self.host = host[7:]
             use_ssl = False
@@ -99,9 +100,9 @@ class Server(object):
             self.host = host
 
         if self.ipc:
-            if str == bytes:  # Python 2
+            if str is bytes:  # Python 2
                 self.host = unquote(host[7:]).decode('utf-8')
-            else:
+            else:  # Python 3
                 self.host = unquote(host[7:])  # encoding defaults to utf-8 in python3
             self.port = None
         elif ':' in self.host and self.host.count(':') == 1:
@@ -198,6 +199,8 @@ class Server(object):
         self.connect_timeout = connect_timeout
         self.mode = mode
 
+        self.get_info_from_server(None)  # load offline schema if needed
+
         if log_enabled(BASIC):
             log(BASIC, 'instantiated Server: <%r>', self)
 
@@ -227,7 +230,8 @@ class Server(object):
 
     @property
     def address_info(self):
-        if not self._address_info or (datetime.now() - self._address_info_resolved_time).seconds > ADDRESS_INFO_REFRESH_TIME:
+        conf_refresh_interval = get_config_parameter('ADDRESS_INFO_REFRESH_TIME')
+        if not self._address_info or (datetime.now() - self._address_info_resolved_time).seconds > conf_refresh_interval:
             # converts addresses tuple to list and adds a 6th parameter for availability (None = not checked, True = available, False=not available) and a 7th parameter for the checking time
             addresses = None
             try:
@@ -276,6 +280,7 @@ class Server(object):
         and port to check availability. Timeout in seconds is specified in CHECK_AVAILABITY_TIMEOUT if not specified in
         the Server object
         """
+        conf_availability_timeout = get_config_parameter('CHECK_AVAILABILITY_TIMEOUT')
         available = False
         self.reset_availability()
         for address in self.candidate_addresses():
@@ -285,7 +290,7 @@ class Server(object):
                 if self.connect_timeout:
                     temp_socket.settimeout(self.connect_timeout)
                 else:
-                    temp_socket.settimeout(get_config_parameter('CHECK_AVAILABILITY_TIMEOUT'))  # set timeout for checking availability to default
+                    temp_socket.settimeout(conf_availability_timeout)  # set timeout for checking availability to default
                 try:
                     temp_socket.connect(address[4])
                 except socket.error:
@@ -329,6 +334,9 @@ class Server(object):
         """
         Retrieve DSE operational attribute as per RFC4512 (5.1).
         """
+        if connection.strategy.no_real_dsa:  # do not try for mock strategies
+            return
+
         if not connection.strategy.pooled:  # in pooled strategies get_dsa_info is performed by the worker threads
             result = connection.search(search_base='',
                                        search_filter='(objectClass=*)',
@@ -364,6 +372,9 @@ class Server(object):
         Retrieve schema from subschemaSubentry DSE attribute, per RFC
         4512 (4.4 and 5.1); entry = '' means DSE.
         """
+        if connection.strategy.no_real_dsa:  # do not try for mock strategies
+            return
+
         schema_entry = None
         if self._dsa_info and entry == '':  # subschemaSubentry already present in dsaInfo
             if isinstance(self._dsa_info.schema_entry, SEQUENCE_TYPES):
@@ -373,13 +384,16 @@ class Server(object):
         else:
             result = connection.search(entry, '(objectClass=*)', BASE, attributes=['subschemaSubentry'], get_operational_attributes=True)
             if isinstance(result, bool):  # sync request
-                schema_entry = connection.response[0]['attributes']['subschemaSubentry'][0] if result else None
+                if result:
+                    schema_entry = connection.response[0]['raw_attributes']['subschemaSubentry'][0]
             else:  # async request, must check if subschemaSubentry in attributes
                 results, _ = connection.get_response(result)
-                if len(results) == 1 and 'attributes' in results[0] and 'subschemaSubentry' in results[0]['attributes']:
-                    schema_entry = results[0]['attributes']['subschemaSubentry'][0]
+                if len(results) == 1 and 'raw_attributes' in results[0] and 'subschemaSubentry' in results[0]['attributes']:
+                    schema_entry = results[0]['raw_attributes']['subschemaSubentry'][0]
 
         if schema_entry and not connection.strategy.pooled:  # in pooled strategies get_schema_info is performed by the worker threads
+            if isinstance(schema_entry, bytes) and str is not bytes:  # Python 3
+                schema_entry = schema_entry.decode('utf-8')
             result = connection.search(schema_entry,
                                        search_filter='(objectClass=subschema)',
                                        search_scope=BASE,
@@ -402,9 +416,11 @@ class Server(object):
                     if isinstance(result, bool):  # sync request
                         self._schema_info = SchemaInfo(schema_entry, connection.response[0]['attributes'], connection.response[0]['raw_attributes']) if result else None
                     else:  # async request, must check if attributes in response
-                        results, _ = connection.get_response(result)
+                        results, result = connection.get_response(result)
                         if len(results) == 1 and 'attributes' in results[0] and 'raw_attributes' in results[0]:
                             self._schema_info = SchemaInfo(schema_entry, results[0]['attributes'], results[0]['raw_attributes'])
+                    if self._schema_info and not self._schema_info.is_valid():  # flaky servers can return an empty schema, checks if it is so and set schema to None
+                        self._schema_info = None
                     if self._schema_info:  # if schema is valid tries to apply formatter to the "other" dict with raw values for schema and info
                         for attribute in self._schema_info.other:
                             self._schema_info.other[attribute] = format_attribute_values(self._schema_info, attribute, self._schema_info.raw[attribute], self.custom_formatter)
@@ -418,28 +434,27 @@ class Server(object):
         """
         reads info from DSE and from subschema
         """
-        if not connection.closed:
-            if self.get_info in [DSA, ALL, OFFLINE_EDIR_8_8_8, OFFLINE_AD_2012_R2, OFFLINE_SLAPD_2_4, OFFLINE_DS389_1_3_3]:
+        if connection and not connection.closed:
+            if self.get_info in [DSA, ALL]:
                 self._get_dsa_info(connection)
-
             if self.get_info in [SCHEMA, ALL]:
                     self._get_schema_info(connection)
-            elif self.get_info == OFFLINE_EDIR_8_8_8:
-                from ..protocol.schemas.edir888 import edir_8_8_8_schema, edir_8_8_8_dsa_info
-                self.attach_schema_info(SchemaInfo.from_json(edir_8_8_8_schema))
-                self.attach_dsa_info(DsaInfo.from_json(edir_8_8_8_dsa_info))
-            elif self.get_info == OFFLINE_AD_2012_R2:
-                from ..protocol.schemas.ad2012R2 import ad_2012_r2_schema, ad_2012_r2_dsa_info
-                self.attach_schema_info(SchemaInfo.from_json(ad_2012_r2_schema))
-                self.attach_dsa_info(DsaInfo.from_json(ad_2012_r2_dsa_info))
-            elif self.get_info == OFFLINE_SLAPD_2_4:
-                from ..protocol.schemas.slapd24 import slapd_2_4_schema, slapd_2_4_dsa_info
-                self.attach_schema_info(SchemaInfo.from_json(slapd_2_4_schema))
-                self.attach_dsa_info(DsaInfo.from_json(slapd_2_4_dsa_info))
-            elif self.get_info == OFFLINE_DS389_1_3_3:
-                from ..protocol.schemas.ds389 import ds389_1_3_3_schema, ds389_1_3_3_dsa_info
-                self.attach_schema_info(SchemaInfo.from_json(ds389_1_3_3_schema))
-                self.attach_dsa_info(DsaInfo.from_json(ds389_1_3_3_dsa_info))
+        elif self.get_info == OFFLINE_EDIR_8_8_8:
+            from ..protocol.schemas.edir888 import edir_8_8_8_schema, edir_8_8_8_dsa_info
+            self.attach_schema_info(SchemaInfo.from_json(edir_8_8_8_schema))
+            self.attach_dsa_info(DsaInfo.from_json(edir_8_8_8_dsa_info))
+        elif self.get_info == OFFLINE_AD_2012_R2:
+            from ..protocol.schemas.ad2012R2 import ad_2012_r2_schema, ad_2012_r2_dsa_info
+            self.attach_schema_info(SchemaInfo.from_json(ad_2012_r2_schema))
+            self.attach_dsa_info(DsaInfo.from_json(ad_2012_r2_dsa_info))
+        elif self.get_info == OFFLINE_SLAPD_2_4:
+            from ..protocol.schemas.slapd24 import slapd_2_4_schema, slapd_2_4_dsa_info
+            self.attach_schema_info(SchemaInfo.from_json(slapd_2_4_schema))
+            self.attach_dsa_info(DsaInfo.from_json(slapd_2_4_dsa_info))
+        elif self.get_info == OFFLINE_DS389_1_3_3:
+            from ..protocol.schemas.ds389 import ds389_1_3_3_schema, ds389_1_3_3_dsa_info
+            self.attach_schema_info(SchemaInfo.from_json(ds389_1_3_3_schema))
+            self.attach_dsa_info(DsaInfo.from_json(ds389_1_3_3_dsa_info))
 
     def attach_dsa_info(self, dsa_info=None):
         if isinstance(dsa_info, DsaInfo):
@@ -466,27 +481,39 @@ class Server(object):
         """
         Define a dummy server with preloaded schema and info
         :param host: host name
-        :param dsa_info: DsaInfo preloaded object
-        :param dsa_schema: SchemaInfo preloaded object
+        :param dsa_info: DsaInfo preloaded object or a json formatted string or a file name
+        :param dsa_schema: SchemaInfo preloaded object or a json formatted string or a file name
         :param port: dummy port
         :param use_ssl: use_ssl
         :param formatter: custom formatter
         :return: Server object
         """
         if isinstance(host, SEQUENCE_TYPES):
-            dummy = Server(host=host[0], port=port, use_ssl=use_ssl, formatter=formatter)  # for ServerPool object
+            dummy = Server(host=host[0], port=port, use_ssl=use_ssl, formatter=formatter, get_info=ALL)  # for ServerPool object
         else:
-            dummy = Server(host=host, port=port, use_ssl=use_ssl, formatter=formatter)
+            dummy = Server(host=host, port=port, use_ssl=use_ssl, formatter=formatter, get_info=ALL)
         if isinstance(dsa_info, DsaInfo):
             dummy._dsa_info = dsa_info
-        else:
+        elif isinstance(dsa_info, STRING_TYPES):
+            try:
+                dummy._dsa_info = DsaInfo.from_json(dsa_info)  # tries to use dsa_info as a json configuration string
+            except Exception:
+                dummy._dsa_info = DsaInfo.from_file(dsa_info)  # tries to use dsa_info as a file name
+
+        if not dummy.info:
             if log_enabled(ERROR):
                 log(ERROR, 'invalid DSA info for %s', host)
             raise LDAPDefinitionError('invalid dsa info')
 
         if isinstance(dsa_schema, SchemaInfo):
             dummy._schema_info = dsa_schema
-        else:
+        elif isinstance(dsa_schema, STRING_TYPES):
+            try:
+                dummy._schema_info = SchemaInfo.from_json(dsa_schema)
+            except Exception:
+                dummy._schema_info = SchemaInfo.from_file(dsa_schema)
+
+        if not dummy.schema:
             if log_enabled(ERROR):
                 log(ERROR, 'invalid schema info for %s', host)
             raise LDAPDefinitionError('invalid schema info')
@@ -497,11 +524,18 @@ class Server(object):
         return dummy
 
     def candidate_addresses(self):
+        conf_reset_availability_timeout = get_config_parameter('RESET_AVAILABILITY_TIMEOUT')
         if self.ipc:
             candidates = self.address_info
             if log_enabled(BASIC):
-               log(BASIC, 'candidate address for <%s>: <%s> with mode UNIX_SOCKET', self, self.name)
+                log(BASIC, 'candidate address for <%s>: <%s> with mode UNIX_SOCKET', self, self.name)
         else:
+            # checks reset availability timeout
+            for address in self.address_info:
+                if address[6] and ((datetime.now() - address[6]).seconds > conf_reset_availability_timeout):
+                    address[5] = None
+                    address[6] = None
+
             # selects server address based on server mode and availability (in address[5])
             addresses = self.address_info[:]  # copy to avoid refreshing while searching candidates
             candidates = []
@@ -514,7 +548,7 @@ class Server(object):
                     candidates = [address for address in addresses if address[0] == socket.AF_INET6 and (address[5] or address[5] is None)]
                 elif self.mode == IP_V4_PREFERRED:
                     candidates = [address for address in addresses if address[0] == socket.AF_INET and (address[5] or address[5] is None)]
-                    candidates += [address for address in addresses if address[0] == socket.AF_INET6 and (addresses[5] or address[5] is None)]
+                    candidates += [address for address in addresses if address[0] == socket.AF_INET6 and (address[5] or address[5] is None)]
                 elif self.mode == IP_V6_PREFERRED:
                     candidates = [address for address in addresses if address[0] == socket.AF_INET6 and (address[5] or address[5] is None)]
                     candidates += [address for address in addresses if address[0] == socket.AF_INET and (address[5] or address[5] is None)]
