@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -25,28 +25,28 @@
 
 from .. import SEQUENCE_TYPES
 from ..protocol.rfc4511 import AddRequest, LDAPDN, AttributeList, Attribute, AttributeDescription, ResultCode, Vals
-from ..protocol.convert import referrals_to_list, attributes_to_dict, validate_attribute_value
-
-# AddRequest ::= [APPLICATION 8] SEQUENCE {
-#     entry           LDAPDN,
-#     attributes      AttributeList }
+from ..protocol.convert import referrals_to_list, attributes_to_dict, validate_attribute_value, prepare_for_sending
 
 
 def add_operation(dn,
                   attributes,
+                  auto_encode,
                   schema=None):
+    # AddRequest ::= [APPLICATION 8] SEQUENCE {
+    #     entry           LDAPDN,
+    #     attributes      AttributeList }
+    #
     # attributes is a dictionary in the form 'attribute': ['val1', 'val2', 'valN']
     attribute_list = AttributeList()
     for pos, attribute in enumerate(attributes):
         attribute_list[pos] = Attribute()
         attribute_list[pos]['type'] = AttributeDescription(attribute)
-        # vals = ValsAtLeast1()
         vals = Vals()  # changed from ValsAtLeast1() for allowing empty member value in groups
         if isinstance(attributes[attribute], SEQUENCE_TYPES):
             for index, value in enumerate(attributes[attribute]):
-                vals.setComponentByPosition(index, validate_attribute_value(schema, attribute, value))
+                vals.setComponentByPosition(index, prepare_for_sending(validate_attribute_value(schema, attribute, value, auto_encode)))
         else:
-            vals.setComponentByPosition(0, validate_attribute_value(schema, attribute, attributes[attribute]))
+            vals.setComponentByPosition(0, prepare_for_sending(validate_attribute_value(schema, attribute, attributes[attribute], auto_encode)))
 
         attribute_list[pos]['vals'] = vals
 
@@ -63,8 +63,8 @@ def add_request_to_dict(request):
 
 
 def add_response_to_dict(response):
-    return {'result': int(response[0]),
-            'description': ResultCode().getNamedValues().getName(response[0]),
+    return {'result': int(response['resultCode']),
+            'description': ResultCode().getNamedValues().getName(response['resultCode']),
             'dn': str(response['matchedDN']),
             'message': str(response['diagnosticMessage']),
             'referrals': referrals_to_list(response['referral'])}
