@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -26,8 +26,7 @@
 from .. import SEQUENCE_TYPES, MODIFY_ADD, MODIFY_DELETE, MODIFY_REPLACE, MODIFY_INCREMENT
 from ..protocol.rfc4511 import ModifyRequest, LDAPDN, Changes, Change, Operation, PartialAttribute, AttributeDescription, Vals, ResultCode
 from ..operation.bind import referrals_to_list
-from ..protocol.convert import changes_to_list, validate_attribute_value
-
+from ..protocol.convert import changes_to_list, validate_attribute_value, prepare_for_sending
 
 # ModifyRequest ::= [APPLICATION 6] SEQUENCE {
 #    object          LDAPDN,
@@ -51,6 +50,7 @@ change_table = {MODIFY_ADD: 0,  # accepts actual values too
 
 def modify_operation(dn,
                      changes,
+                     auto_encode,
                      schema=None):
     # changes is a dictionary in the form {'attribute': [(operation, [val1, ...]), ...], ...}
     # operation is 0 (add), 1 (delete), 2 (replace), 3 (increment)
@@ -65,10 +65,9 @@ def modify_operation(dn,
             partial_attribute['vals'] = Vals()
             if isinstance(change_operation[1], SEQUENCE_TYPES):
                 for index, value in enumerate(change_operation[1]):
-                    partial_attribute['vals'].setComponentByPosition(index, validate_attribute_value(schema, attribute, value))
+                    partial_attribute['vals'].setComponentByPosition(index, prepare_for_sending(validate_attribute_value(schema, attribute, value, auto_encode)))
             else:
-                partial_attribute['vals'].setComponentByPosition(0, validate_attribute_value(schema, attribute, change_operation[1]))
-
+                partial_attribute['vals'].setComponentByPosition(0, prepare_for_sending(validate_attribute_value(schema, attribute, change_operation[1], auto_encode)))
             change = Change()
             change['operation'] = Operation(change_table[change_operation[0]])
             change['modification'] = partial_attribute
@@ -79,7 +78,6 @@ def modify_operation(dn,
     request = ModifyRequest()
     request['object'] = LDAPDN(dn)
     request['changes'] = change_list
-
     return request
 
 
@@ -89,7 +87,8 @@ def modify_request_to_dict(request):
 
 
 def modify_response_to_dict(response):
-    return {'result': int(response[0]),
-            'description': ResultCode().getNamedValues().getName(response[0]),
-            'message': str(response['diagnosticMessage']), 'dn': str(response['matchedDN']),
+    return {'result': int(response['resultCode']),
+            'description': ResultCode().getNamedValues().getName(response['resultCode']),
+            'message': str(response['diagnosticMessage']),
+            'dn': str(response['matchedDN']),
             'referrals': referrals_to_list(response['referral'])}
