@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -25,16 +25,16 @@
 
 from binascii import hexlify
 from uuid import UUID
-from datetime import datetime
+from datetime import datetime, timedelta
 
 from ...core.timezone import OffsetTzInfo
 
 
 def format_unicode(raw_value):
     try:
-        if str != bytes:  # python3
+        if str is not bytes:  # Python 3
             return str(raw_value, 'utf-8', errors='strict')
-        else:
+        else:  # Python 2
             return unicode(raw_value, 'utf-8', errors='strict')
     except (TypeError, UnicodeDecodeError):
         pass
@@ -90,16 +90,20 @@ def format_boolean(raw_value):
 
 def format_ad_timestamp(raw_value):
     """
-    The Active Directory stores date/time values as the number of 100-nanosecond intervals
+    Active Directory stores date/time values as the number of 100-nanosecond intervals
     that have elapsed since the 0 hour on January 1, 1601 till the date/time that is being stored.
     The time is always stored in Greenwich Mean Time (GMT) in the Active Directory.
     """
+    if raw_value == b'9223372036854775807':  # max value to be stored in a 64 bit signed int
+        return datetime.max  # returns datetime.datetime(9999, 12, 31, 23, 59, 59, 999999)
     try:
         timestamp = int(raw_value)
         return datetime.fromtimestamp(timestamp / 10000000.0 - 11644473600, tz=OffsetTzInfo(0, 'UTC'))  # forces true division in python 2
-    except Exception:
-        if raw_value == b'9223372036854775807':  # max value to be stored in a 64 bit signed int
-            return datetime.max  # returns datetime.datetime(9999, 12, 31, 23, 59, 59, 999999)
+    except (OSError, OverflowError, ValueError):  # on Windows backwards timestamps are not allowed
+        unix_epoch = datetime.fromtimestamp(0, tz=OffsetTzInfo(0, 'UTC'))
+        diff_seconds = timedelta(seconds=timestamp/10000000.0 - 11644473600)
+        return unix_epoch + diff_seconds
+    except Exception as e:
         return raw_value
 
 
@@ -110,7 +114,7 @@ def format_time(raw_value):
     '''
     From RFC4517:
     A value of the Generalized Time syntax is a character string
-    representing a date and time.  The LDAP-specific encoding of a value
+    representing a date and time. The LDAP-specific encoding of a value
     of this syntax is a restriction of the format defined in [ISO8601],
     and is described by the following ABNF:
 
@@ -136,6 +140,7 @@ def format_time(raw_value):
     g-differential  = ( MINUS / PLUS ) hour [ minute ]
         MINUS           = %x2D  ; minus sign ("-")
     '''
+    # if len(raw_value) < 10 or not all((c in b'0123456789+-,.Z' for c in raw_value)) or (b'Z' in raw_value and not raw_value.endswith(b'Z')):  # first ten characters are mandatory and must be numeric or timezone or fraction
     if len(raw_value) < 10 or not all((c in b'0123456789+-,.Z' for c in raw_value)) or (b'Z' in raw_value and not raw_value.endswith(b'Z')):  # first ten characters are mandatory and must be numeric or timezone or fraction
         return raw_value
 
@@ -164,14 +169,14 @@ def format_time(raw_value):
     if time and (b'.' in time or b',' in time):
         # fraction time
         if time[0] in b',.':
-            minute = 6 * int(time[1] if str == bytes else chr(time[1]))
+            minute = 6 * int(time[1] if str is bytes else chr(time[1]))  # Python 2 / Python 3
         elif time[2] in b',.':
             minute = int(raw_value[10: 12])
-            second = 6 * int(time[3] if str == bytes else chr(time[3]))
+            second = 6 * int(time[3] if str is bytes else chr(time[3]))  # Python 2 / Python 3
         elif time[4] in b',.':
             minute = int(raw_value[10: 12])
             second = int(raw_value[12: 14])
-            microsecond = 100000 * int(time[5] if str == bytes else chr(time[5]))
+            microsecond = 100000 * int(time[5] if str is bytes else chr(time[5]))  # Python 2 / Python 3
     elif len(time) == 2:  # mmZ format
         minute = int(raw_value[10: 12])
     elif len(remain) == 0:  # Z format
@@ -196,9 +201,12 @@ def format_time(raw_value):
                 raise ValueError
         except ValueError:
             return raw_value
-        if str != bytes:  # python3
+        if timezone_hour > 23 or timezone_minute > 59:  # invalid timezone
+            return raw_value
+
+        if str is not bytes:  # Python 3
             timezone = OffsetTzInfo((timezone_hour * 60 + timezone_minute) * (1 if sep == b'+' else -1), 'UTC' + str(sep + offset, encoding='utf-8'))
-        else:
+        else:  # Python 2
             timezone = OffsetTzInfo((timezone_hour * 60 + timezone_minute) * (1 if sep == b'+' else -1), unicode('UTC' + sep + offset, encoding='utf-8'))
 
     try:
@@ -245,7 +253,7 @@ def format_sid(raw_value):
     SubAuthority (variable): A variable length array of unsigned 32-bit integers that uniquely identifies a principal relative to the IdentifierAuthority. Its length is determined by SubAuthorityCount.
     '''
 
-    if str != bytes:  # python 3
+    if str is not bytes:  # Python 3
         revision = int(raw_value[0])
         sub_authority_count = int(raw_value[1])
         identifier_authority = int.from_bytes(raw_value[2:8], byteorder='big')
@@ -257,7 +265,7 @@ def format_sid(raw_value):
         while i < sub_authority_count:
             sub_authority += '-' + str(int.from_bytes(raw_value[8 + (i * 4): 12 + (i * 4)], byteorder='little'))  # little endian
             i += 1
-    else:  # python 2
+    else:  # Python 2
         revision = int(ord(raw_value[0]))
         sub_authority_count = int(ord(raw_value[1]))
         identifier_authority = int(hexlify(raw_value[2:8]), 16)
