@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -44,12 +44,10 @@ from pyasn1.type.constraint import ValueRangeConstraint, SingleValueConstraint,
 from pyasn1.type.namedval import NamedValues
 from pyasn1.type.tag import tagClassApplication, tagFormatConstructed, Tag, tagClassContext, tagFormatSimple
 
-from .. import LDAP_MAX_INT
-
 
 # constants
 # maxInt INTEGER ::= 2147483647 -- (2^^31 - 1) --
-
+LDAP_MAX_INT = 2147483647
 MAXINT = Integer(LDAP_MAX_INT)
 
 # constraints
@@ -371,7 +369,7 @@ class Referral(SequenceOf):
 class ServerSaslCreds(OctetString):
     # serverSaslCreds    [7] OCTET STRING OPTIONAL
     tagSet = OctetString.tagSet.tagImplicitly(Tag(tagClassContext, tagFormatSimple, 7))
-    # encoding = 'utf-8'
+    encoding = 'utf-8'
 
 
 class LDAPResult(Sequence):
@@ -464,7 +462,9 @@ class Scope(Enumerated):
     #     baseObject              (0),
     #     singleLevel             (1),
     #     wholeSubtree            (2),
-    namedValues = NamedValues(('baseObject', 0), ('singleLevel', 1), ('wholeSubtree', 2))
+    namedValues = NamedValues(('baseObject', 0),
+                              ('singleLevel', 1),
+                              ('wholeSubtree', 2))
 
 
 class DerefAliases(Enumerated):
@@ -473,7 +473,10 @@ class DerefAliases(Enumerated):
     #     derefInSearching        (1),
     #     derefFindingBaseObj     (2),
     #     derefAlways             (3) },
-    namedValues = NamedValues(('neverDerefAliases', 0), ('derefInSearching', 1), ('derefFindingBaseObj', 2), ('derefAlways', 3))
+    namedValues = NamedValues(('neverDerefAliases', 0),
+                              ('derefInSearching', 1),
+                              ('derefFindingBaseObj', 2),
+                              ('derefAlways', 3))
 
 
 class TypesOnly(Boolean):
@@ -666,7 +669,10 @@ class Operation(Enumerated):
     #     delete  (1),
     #     replace (2),
     #     ...  }
-    namedValues = NamedValues(('add', 0), ('delete', 1), ('replace', 2), ('increment', 3))
+    namedValues = NamedValues(('add', 0),
+                              ('delete', 1),
+                              ('replace', 2),
+                              ('increment', 3))
 
 
 class Change(Sequence):
