@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -30,7 +30,7 @@ import json
 from .oid import CLASS_ABSTRACT, CLASS_STRUCTURAL, CLASS_AUXILIARY, ATTRIBUTE_USER_APPLICATION, \
     ATTRIBUTE_DIRECTORY_OPERATION, ATTRIBUTE_DISTRIBUTED_OPERATION, ATTRIBUTE_DSA_OPERATION
 from .. import SEQUENCE_TYPES, STRING_TYPES, get_config_parameter
-from ..utils.conv import escape_bytes, json_hook, check_json_dict, format_json, check_escape
+from ..utils.conv import escape_bytes, json_hook, check_json_dict, format_json, to_unicode
 from ..utils.ciDict import CaseInsensitiveDict
 from ..protocol.formatters.standard import format_attribute_values
 from .oid import Oids, decode_oids, decode_syntax, oid_to_string
@@ -45,7 +45,7 @@ def constant_to_class_kind(value):
     elif value == CLASS_AUXILIARY:
         return 'Auxiliary'
     else:
-        return 'unknown'
+        return '<unknown>'
 
 
 def constant_to_attribute_usage(value):
@@ -82,7 +82,8 @@ def quoted_string_to_list(quoted_string):
     if string[0] == '(' and string[-1] == ')':
         string = string[1:-1]
     elements = string.split("'")
-    return [check_escape(element.strip("'").strip()) for element in elements if element.strip()]
+    # return [check_escape(element.strip("'").strip()) for element in elements if element.strip()]
+    return [element.strip("'").strip() for element in elements if element.strip()]
 
 
 def oids_string_to_list(oid_string):
@@ -116,21 +117,24 @@ class BaseServerInfo(object):
 
     @classmethod
     def from_json(cls, json_definition, schema=None, custom_formatter=None):
+        conf_case_insensitive_schema = get_config_parameter('CASE_INSENSITIVE_SCHEMA_NAMES')
         definition = json.loads(json_definition, object_hook=json_hook)
         if 'raw' not in definition or 'type' not in definition:
             raise LDAPDefinitionError('invalid JSON definition')
 
-        if get_config_parameter('CASE_INSENSITIVE_SCHEMA_NAMES'):
+        if conf_case_insensitive_schema:
             attributes = CaseInsensitiveDict()
         else:
             attributes = dict()
 
         if schema:
             for attribute in definition['raw']:
-                attributes[attribute] = format_attribute_values(schema, check_escape(attribute), check_escape(definition['raw'][attribute]), custom_formatter)
+                # attributes[attribute] = format_attribute_values(schema, check_escape(attribute), [check_escape(value) for value in definition['raw'][attribute]], custom_formatter)
+                attributes[attribute] = format_attribute_values(schema, attribute, [value for value in definition['raw'][attribute]], custom_formatter)
         else:
             for attribute in definition['raw']:
-                attributes[attribute] = check_escape(definition['raw'][attribute])
+                # attributes[attribute] = [check_escape(value) for value in definition['raw'][attribute]]
+                attributes[attribute] = [value for value in definition['raw'][attribute]]
 
         if cls.__name__ != definition['type']:
             raise LDAPDefinitionError('JSON info not of type ' + cls.__name__)
@@ -180,7 +184,7 @@ class BaseServerInfo(object):
         else:
             raise LDAPDefinitionError('unable to convert ' + str(self) + ' to JSON')
 
-        if str == bytes:  # python 2
+        if str is bytes:  # Python 2
             check_json_dict(json_dict)
 
         return json.dumps(json_dict, ensure_ascii=False, sort_keys=sort, indent=indent, check_circular=True, default=format_json, separators=(',', ': '))
@@ -209,45 +213,45 @@ class DsaInfo(BaseServerInfo):
     def __repr__(self):
         r = 'DSA info (from DSE):' + linesep
         if isinstance(self.supported_ldap_versions, SEQUENCE_TYPES):
-            r += ('  Supported LDAP Versions: ' + ', '.join([str(s) for s in self.supported_ldap_versions])) if self.supported_ldap_versions else ''
+            r += ('  Supported LDAP versions: ' + ', '.join([str(s) for s in self.supported_ldap_versions])) if self.supported_ldap_versions else ''
         else:
-            r += ('  Supported LDAP Versions: ' + str(self.supported_ldap_versions))
+            r += ('  Supported LDAP versions: ' + str(self.supported_ldap_versions))
         r += linesep
         if isinstance(self.naming_contexts, SEQUENCE_TYPES):
-            r += ('  Naming Contexts: ' + linesep + linesep.join(['    ' + str(s) for s in self.naming_contexts])) if self.naming_contexts else ''
+            r += ('  Naming contexts: ' + linesep + linesep.join(['    ' + str(s) for s in self.naming_contexts])) if self.naming_contexts else ''
         else:
-            r += ('  Naming Contexts: ' + str(self.naming_contexts))
+            r += ('  Naming contexts: ' + str(self.naming_contexts))
         r += linesep
         if isinstance(self.alt_servers, SEQUENCE_TYPES):
-            r += ('  Alternative Servers: ' + linesep + linesep.join(['    ' + str(s) for s in self.alt_servers])) if self.alt_servers else ''
+            r += ('  Alternative servers: ' + linesep + linesep.join(['    ' + str(s) for s in self.alt_servers])) if self.alt_servers else ''
         else:
-            r += ('  Alternative Servers: ' + str(self.alt_servers))
+            r += ('  Alternative servers: ' + str(self.alt_servers))
         r += linesep
         if isinstance(self.supported_controls, SEQUENCE_TYPES):
-            r += ('  Supported Controls: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_controls])) if self.supported_controls else ''
+            r += ('  Supported controls: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_controls])) if self.supported_controls else ''
         else:
-            r += ('  Supported Controls: ' + str(self.supported_controls))
+            r += ('  Supported controls: ' + str(self.supported_controls))
         r += linesep
         if isinstance(self.supported_extensions, SEQUENCE_TYPES):
-            r += ('  Supported Extensions: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_extensions])) if self.supported_extensions else ''
+            r += ('  Supported extensions: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_extensions])) if self.supported_extensions else ''
         else:
-            r += ('  Supported Extensions: ' + str(self.supported_extensions))
+            r += ('  Supported extensions: ' + str(self.supported_extensions))
         r += linesep
         if self.supported_features:
             if isinstance(self.supported_features, SEQUENCE_TYPES):
-                r += ('  Supported Features: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_features])) if self.supported_features else ''
+                r += ('  Supported features: ' + linesep + linesep.join(['    ' + oid_to_string(s) for s in self.supported_features])) if self.supported_features else ''
             else:
-                r += ('  Supported Features: ' + str(self.supported_features))
+                r += ('  Supported features: ' + str(self.supported_features))
             r += linesep
         if isinstance(self.supported_sasl_mechanisms, SEQUENCE_TYPES):
-            r += ('  Supported SASL Mechanisms: ' + linesep + '    ' + ', '.join([str(s) for s in self.supported_sasl_mechanisms])) if self.supported_sasl_mechanisms else ''
+            r += ('  Supported SASL mechanisms: ' + linesep + '    ' + ', '.join([str(s) for s in self.supported_sasl_mechanisms])) if self.supported_sasl_mechanisms else ''
         else:
-            r += ('  Supported SASL Mechanisms: ' + str(self.supported_sasl_mechanisms))
+            r += ('  Supported SASL mechanisms: ' + str(self.supported_sasl_mechanisms))
         r += linesep
         if isinstance(self.schema_entry, SEQUENCE_TYPES):
-            r += ('  Schema Entry: ' + linesep + linesep.join(['    ' + str(s) for s in self.schema_entry])) if self.schema_entry else ''
+            r += ('  Schema entry: ' + linesep + linesep.join(['    ' + str(s) for s in self.schema_entry])) if self.schema_entry else ''
         else:
-            r += ('  Schema Entry: ' + str(self.schema_entry))
+            r += ('  Schema entry: ' + str(self.schema_entry))
         r += linesep
         if isinstance(self.vendor_name, SEQUENCE_TYPES) and len(self.vendor_name) == 1:
             r += 'Vendor name: ' + self.vendor_name[0]
@@ -291,18 +295,25 @@ class SchemaInfo(BaseServerInfo):
         self.ldap_syntaxes = LdapSyntaxInfo.from_definition(attributes.pop('ldapSyntaxes', []))
         self.other = attributes  # remaining schema definition attributes not in RFC4512
 
-        # links attributes to objects
-        for object_class in self.object_classes:
-            for attribute in self.object_classes[object_class].must_contain:
-                try:
-                    self.attribute_types[attribute].mandatory_in.append(object_class)
-                except KeyError:
-                    pass
-            for attribute in self.object_classes[object_class].may_contain:
-                try:
-                    self.attribute_types[attribute].optional_in.append(object_class)
-                except KeyError:
-                    pass
+        # links attributes to class objects
+        if self.object_classes and self.attribute_types:
+            for object_class in self.object_classes:  # CaseInsensitiveDict return keys while iterating
+                for attribute in self.object_classes[object_class].must_contain:
+                    try:
+                        self.attribute_types[attribute].mandatory_in.append(object_class)
+                    except KeyError:
+                        pass
+                for attribute in self.object_classes[object_class].may_contain:
+                    try:
+                        self.attribute_types[attribute].optional_in.append(object_class)
+                    except KeyError:
+                        pass
+
+    def is_valid(self):
+        if self.object_classes or self.attribute_types or self.matching_rules or self.matching_rule_uses or self.dit_content_rules or self.dit_structure_rules or self.name_forms or self.ldap_syntaxes:
+            return True
+        return False
+
     def __repr__(self):
         r = 'DSA Schema from: ' + self.schema_entry
         r += linesep
@@ -405,12 +416,17 @@ class BaseObjectInfo(object):
 
     @classmethod
     def from_definition(cls, definitions):
+        conf_case_insensitive_schema = get_config_parameter('CASE_INSENSITIVE_SCHEMA_NAMES')
+        conf_ignore_malformed_schema = get_config_parameter('IGNORE_MALFORMED_SCHEMA')
+
+        ret_dict = CaseInsensitiveDict() if conf_case_insensitive_schema else dict()
+
         if not definitions:
-            return None
+            return CaseInsensitiveDict() if conf_case_insensitive_schema else dict()
 
-        ret_dict = CaseInsensitiveDict() if get_config_parameter('CASE_INSENSITIVE_SCHEMA_NAMES') else dict()
         for object_definition in definitions:
-            if [object_definition[0] == ')' and object_definition[:-1] == ')']:
+            object_definition = to_unicode(object_definition.strip(), additional_encodings=True)
+            if object_definition[0] == '(' and object_definition[-1] == ')':
                 if cls is MatchingRuleInfo:
                     pattern = '| SYNTAX '
                 elif cls is ObjectClassInfo:
@@ -419,8 +435,6 @@ class BaseObjectInfo(object):
                     pattern = '| SUP | EQUALITY | ORDERING | SUBSTR | SYNTAX | SINGLE-VALUE| COLLECTIVE| NO-USER-MODIFICATION| USAGE '
                 elif cls is MatchingRuleUseInfo:
                     pattern = '| APPLIES '
-                elif cls is DitContentRuleInfo:
-                    pattern = '| AUX '
                 elif cls is LdapSyntaxInfo:
                     pattern = ''
                 elif cls is DitContentRuleInfo:
@@ -494,7 +508,10 @@ class BaseObjectInfo(object):
                             object_def.experimental = []
                         object_def.experimental.append(extension_to_tuple('E-' + value))
                     else:
-                        raise LDAPSchemaError('malformed schema definition key:' + key)
+                        if not conf_ignore_malformed_schema:
+                            raise LDAPSchemaError('malformed schema definition key:' + key + ' - use get_info=NONE in Server definition')
+                        else:
+                            return CaseInsensitiveDict() if conf_case_insensitive_schema else dict()
                 object_def.raw_definition = object_definition
                 if hasattr(object_def, 'syntax') and object_def.syntax and len(object_def.syntax) == 1:
                     object_def.min_length = None
@@ -515,7 +532,10 @@ class BaseObjectInfo(object):
                     ret_dict[object_def.oid] = object_def
 
             else:
-                raise LDAPSchemaError('malformed schema definition')
+                if not conf_ignore_malformed_schema:
+                    raise LDAPSchemaError('malformed schema definition, use get_info=NONE in Server definition')
+                else:
+                    return CaseInsensitiveDict() if conf_case_insensitive_schema else dict()
         return ret_dict
 
 
@@ -611,10 +631,11 @@ class ObjectClassInfo(BaseObjectInfo):
 
     def __repr__(self):
         r = ''
-        r += (linesep + '  Type: ' + constant_to_class_kind(self.kind)) if isinstance(self.kind, int) else ''
+        r += (linesep + '  Type: ' + constant_to_class_kind(self.kind)) if self.kind else ''
+        r += (linesep + '  Superior: ' + list_to_string(self.superior)) if self.superior else ''
         r += (linesep + '  Must contain attributes: ' + list_to_string(self.must_contain)) if self.must_contain else ''
         r += (linesep + '  May contain attributes: ' + list_to_string(self.may_contain)) if self.may_contain else ''
-        return 'Object Class' + BaseObjectInfo.__repr__(self).replace('<__desc__>', r)
+        return 'Object class' + BaseObjectInfo.__repr__(self).replace('<__desc__>', r)
 
 
 class AttributeTypeInfo(BaseObjectInfo):
@@ -664,15 +685,16 @@ class AttributeTypeInfo(BaseObjectInfo):
 
     def __repr__(self):
         r = ''
-        r += linesep + '  Single Value: True' if self.single_value else ''
+        r += linesep + '  Single value: ' + str(self.single_value)
         r += linesep + '  Collective: True' if self.collective else ''
+        r += (linesep + '  Superior: ' + list_to_string(self.superior)) if self.superior else ''
         r += linesep + '  No user modification: True' if self.no_user_modification else ''
         r += (linesep + '  Usage: ' + constant_to_attribute_usage(self.usage)) if self.usage else ''
         r += (linesep + '  Equality rule: ' + list_to_string(self.equality)) if self.equality else ''
         r += (linesep + '  Ordering rule: ' + list_to_string(self.ordering)) if self.ordering else ''
         r += (linesep + '  Substring rule: ' + list_to_string(self.substring)) if self.substring else ''
         r += (linesep + '  Syntax: ' + (self.syntax + (' [' + str(decode_syntax(self.syntax)))) + ']') if self.syntax else ''
-        r += (linesep + '  Minimum Length: ' + str(self.min_length)) if isinstance(self.min_length, int) else ''
+        r += (linesep + '  Minimum length: ' + str(self.min_length)) if isinstance(self.min_length, int) else ''
         r += linesep + '  Mandatory in: ' + list_to_string(self.mandatory_in) if self.mandatory_in else ''
         r += linesep + '  Optional in: ' + list_to_string(self.optional_in) if self.optional_in else ''
         return 'Attribute type' + BaseObjectInfo.__repr__(self).replace('<__desc__>', r)
@@ -691,7 +713,8 @@ class LdapSyntaxInfo(BaseObjectInfo):
                  definition=None):
 
         BaseObjectInfo.__init__(self,
-                                oid=oid, name=None,
+                                oid=oid,
+                                name=None,
                                 description=description,
                                 obsolete=False,
                                 extensions=extensions,
@@ -805,7 +828,7 @@ class NameFormInfo(BaseObjectInfo):
         self.may_contain = may_contain
 
     def __repr__(self):
-        r = (linesep + '  Object class: ' + self.object_class) if self.object_class else ''
+        r = (linesep + '  Object class: ' + list_to_string(self.object_class)) if self.object_class else ''
         r += (linesep + '  Must contain: ' + list_to_string(self.must_contain)) if self.must_contain else ''
         r += (linesep + '  May contain: ' + list_to_string(self.may_contain)) if self.may_contain else ''
         return 'DIT content rule' + BaseObjectInfo.__repr__(self).replace('<__desc__>', r)
