@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2014, 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -25,25 +25,76 @@
 
 from base64 import b64encode, b64decode
 import datetime
+import re
 
-from .. import SEQUENCE_TYPES
+from .. import SEQUENCE_TYPES, STRING_TYPES, NUMERIC_TYPES, get_config_parameter
 from ..utils.ciDict import CaseInsensitiveDict
 from ..core.exceptions import LDAPDefinitionError
 
 
-def escape_filter_chars(text):
+def to_unicode(obj, encoding=None, additional_encodings=False):
+    """Try to convert bytes (and str in python2) to unicode.
+     Return object unmodified if python3 string, else raise an exception
+    """
+    conf_default_encoding = get_config_parameter('DEFAULT_ENCODING')
+    conf_additional_encodings = get_config_parameter('ADDITIONAL_ENCODINGS')
+    if isinstance(obj, NUMERIC_TYPES):
+        obj = str(obj)
+
+    if isinstance(obj, (bytes, bytearray)):
+        if encoding is None:
+            encoding = conf_default_encoding
+        try:
+            return obj.decode(encoding)
+        except UnicodeDecodeError:
+            if additional_encodings:
+                for encoding in conf_additional_encodings:  # AD could have DN not encoded in utf-8 (even if this is not allowed by RFC4510)
+                    try:
+                        return obj.decode(encoding)
+                    except UnicodeDecodeError:
+                        pass
+
+    if isinstance(obj, STRING_TYPES):  # python3 strings, python 2 unicode
+        return obj
+
+    raise UnicodeError("Unable to convert to unicode %r" % obj)
+
+
+def to_raw(obj, encoding='utf-8'):
+    """Tries to convert to raw bytes from unicode"""
+    if isinstance(obj, NUMERIC_TYPES):
+        obj = str(obj)
+
+    if not (isinstance(obj, bytes)):
+        if isinstance(obj, SEQUENCE_TYPES):
+            return [to_raw(element) for element in obj]
+        elif isinstance(obj, STRING_TYPES):
+            return obj.encode(encoding)
+
+    return obj
+
+
+def escape_filter_chars(text, encoding=None):
     """ Escape chars mentioned in RFC4515. """
-    output = text.replace('\\', r'\5c')
-    output = output.replace(r'*', r'\2a')
-    output = output.replace(r'(', r'\28')
-    output = output.replace(r')', r'\29')
-    output = output.replace('\x00', r'\00')
-    return output
+
+    if encoding is None:
+        encoding = get_config_parameter('DEFAULT_ENCODING')
+
+    text = to_unicode(text, encoding)
+    escaped = text.replace('\\', '\\5c')
+    escaped = escaped.replace('*', '\\2a')
+    escaped = escaped.replace('(', '\\28')
+    escaped = escaped.replace(')', '\\29')
+    escaped = escaped.replace('\x00', '\\00')
+    # escape all octets greater than 0x7F that are not part of a valid UTF-8
+    # escaped = ''.join(c if c <= '\x7f' else escape_bytes(to_raw(to_unicode(c, encoding))) for c in output)
+    return escaped
 
 
 def escape_bytes(bytes_value):
+    """ Convert a byte sequence to a properly escaped for LDAP (format BACKSLASH HEX HEX) string"""
     if bytes_value:
-        if str != bytes:  # Python 3
+        if str is not bytes:  # Python 3
             if isinstance(bytes_value, str):
                 bytes_value = bytearray(bytes_value, encoding='utf-8')
             escaped = '\\'.join([('%02x' % int(b)) for b in bytes_value])
@@ -58,31 +109,31 @@ def escape_bytes(bytes_value):
 
 
 def prepare_for_stream(value):
-    if str != bytes:  # Python 3
+    if str is not bytes:  # Python 3
         return value
     else:  # Python 2
         return value.decode()
 
 
-def check_escape(raw_string):
-    if '\\' not in raw_string:
-        return raw_string
-
-    escaped = ''
-    i = 0
-    while i < len(raw_string):
-        if raw_string[i] == '\\' and i < len(raw_string) - 2:
-            try:
-                value = int(raw_string[i + 1: i + 3], 16)
-                escaped += chr(value)
-                i += 2
-            except ValueError:
-                escaped += '\\\\'
-        else:
-            escaped += raw_string[i]
-        i += 1
-
-    return escaped
+# def check_escape(raw_string):
+#     if isinstance(raw_string, bytes) or '\\' not in raw_string:
+#         return raw_string
+#
+#     escaped = ''
+#     i = 0
+#     while i < len(raw_string):
+#         if raw_string[i] == '\\' and i < len(raw_string) - 2:
+#             try:
+#                 value = int(raw_string[i + 1: i + 3], 16)
+#                 escaped += chr(value)
+#                 i += 2
+#             except ValueError:
+#                 escaped += '\\\\'
+#         else:
+#             escaped += raw_string[i]
+#         i += 1
+#
+#     return escaped
 
 
 def json_encode_b64(obj):
@@ -131,20 +182,22 @@ def format_json(obj):
     if isinstance(obj, int):
         return obj
 
-    if str == bytes:
+    if str is bytes:  # Python 2
         if isinstance(obj, long):  # long exists only in python2
             return obj
 
     try:
-        if str != bytes:  # python3
+        if str is not bytes:  # Python 3
             if isinstance(obj, bytes):
-                return check_escape(str(obj, 'utf-8', errors='strict'))
+                # return check_escape(str(obj, 'utf-8', errors='strict'))
+                return str(obj, 'utf-8', errors='strict')
             raise LDAPDefinitionError('unable to serialize ' + str(obj))
-        else:  # python2
+        else:  # Python 2
             if isinstance(obj, unicode):
                 return obj
             else:
-                return unicode(check_escape(obj))
+                # return unicode(check_escape(obj))
+                return unicode(obj)
     except (TypeError, UnicodeDecodeError):
         pass
 
@@ -154,3 +207,10 @@ def format_json(obj):
         pass
 
     raise LDAPDefinitionError('unable to serialize ' + str(obj))
+
+
+def is_filter_escaped(text):
+    if not type(text) == ((str is not bytes) and str or unicode):  # requires str for Python 3 and unicode for Python 2
+        raise ValueError('unicode input expected')
+
+    return all(c not in text for c in '()*\0') and not re.search('\\\\([^0-9a-fA-F]|(.[^0-9a-fA-F]))', text)
