@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -36,7 +36,7 @@ from ..core.exceptions import LDAPInvalidHashAlgorithmError
 
 algorithms_table = {
     HASHED_MD5: ('md5', 'MD5'),
-    HASHED_SHA: ('sha', 'DSA-SHA'),
+    HASHED_SHA: ('sha', 'SHA1'),
     HASHED_SHA256: ('sha256', 'SHA256'),
     HASHED_SHA384: ('sha384', 'SHA384'),
     HASHED_SHA512: ('sha512', 'SHA512')
@@ -53,7 +53,7 @@ salted_table = {
 
 
 def hashed(algorithm, value, salt=None, raw=False, encoding='utf-8'):
-    if str != bytes and not isinstance(value, bytes):  # python 3
+    if str is not bytes and not isinstance(value, bytes):  # Python 3
         value = value.encode(encoding)
 
     if algorithm is None or algorithm == HASHED_NONE:
