@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,8 +23,7 @@
 # along with ldap3 in the COPYING and COPYING.LESSER files.
 # If not, see <http://www.gnu.org/licenses/>.
 
-from logging import getLogger, getLevelName, DEBUG
-from os import linesep
+from logging import getLogger, DEBUG
 from copy import deepcopy
 from pprint import pformat
 from ..protocol.rfc4511 import LDAPMessage
@@ -37,8 +36,10 @@ PROTOCOL = 30
 NETWORK = 40
 EXTENDED = 50
 
-_sensitive_lines = ('simple', 'credentials', 'serversaslcreds')  # must be a tuple, not a list
+_sensitive_lines = ('simple', 'credentials', 'serversaslcreds')  # must be a tuple, not a list, lowercase
 _sensitive_args = ('simple', 'password', 'sasl_credentials', 'saslcreds', 'server_creds')
+_sensitive_attrs = ('userpassword', 'unicodepwd')
+
 _hide_sensitive_data = None
 
 DETAIL_LEVELS = [OFF, ERROR, BASIC, PROTOCOL, NETWORK, EXTENDED]
@@ -70,7 +71,7 @@ def _strip_sensitive_data_from_dict(d):
 
     try:
         d = deepcopy(d)
-    except Exception:  # if deepcopy goes wrong gives up and return the dict unchanged
+    except Exception:  # if deepcopy goes wrong gives up and returns the dict unchanged
         return d
     for k in d.keys():
         if isinstance(d[k], dict):
@@ -103,7 +104,7 @@ def log(detail, message, *args):
             args = tuple([_strip_sensitive_data_from_dict(arg) if isinstance(arg, dict) else arg for arg in args])
 
         encoded_message = (get_detail_level_name(detail) + ':' + message % args).encode(_logging_encoding, 'backslashreplace')
-        if str != bytes:  # Python 3
+        if str is not bytes:  # Python 3
             encoded_message = encoded_message.decode()
 
         if len(encoded_message) > _max_line_length:
@@ -182,24 +183,21 @@ def format_ldap_message(message, prefix):
     prefixed = ''
     for line in (message.prettyPrint().split('\n') if isinstance(message, LDAPMessage) else pformat(message).split('\n')):  # uses pyasn1 LDAP message prettyPrint() method
         if line:
-            if _hide_sensitive_data and line.strip().lower().startswith(_sensitive_lines):  # _sensitive_lines is a tuple. startswith() method check each tuple element
+            if _hide_sensitive_data and line.strip().lower().startswith(_sensitive_lines):  # _sensitive_lines is a tuple. startswith() method checks each tuple element
                 tag, _, data = line.partition('=')
                 if data.startswith("b'") and data.endswith("'") or data.startswith('b"') and data.endswith('"'):
-                    prefixed += linesep + prefix + tag + '=<stripped %d characters of sensitive data>' % (len(data) - 3, )
+                    prefixed += '\n' + prefix + tag + '=<stripped %d characters of sensitive data>' % (len(data) - 3, )
                 else:
-                    prefixed += linesep + prefix + tag + '=<stripped %d characters of sensitive data>' % len(data)
+                    prefixed += '\n' + prefix + tag + '=<stripped %d characters of sensitive data>' % len(data)
             else:
-                prefixed += linesep + prefix + line
+                prefixed += '\n' + prefix + line
     return prefixed
 
 # sets a logger for the library with NullHandler. It can be used by the application with its own logging configuration
 logger = getLogger('ldap3')
 logger.addHandler(NullHandler())
 
-# set defaults for the library logging
+# sets defaults for the library logging
 set_library_log_activation_level(DEBUG)
 set_library_log_detail_level(OFF)
 set_library_log_hide_sensitive_data(True)
-
-# emits a info message to let the application know that ldap3 logging is available when the log level is set to _logging_level
-logger.info('ldap3 library initialized - logging emitted with loglevel set to ' + getLevelName(_logging_level) + ' - available detail levels are: ' + ', '.join([get_detail_level_name(level) for level in DETAIL_LEVELS]) + ' - sensitive data will ' + ('' if _hide_sensitive_data else 'not ') + 'be hidden')
