@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -31,11 +31,11 @@ from socket import gethostname
 from time import time
 import hmac
 import hashlib
+import binascii
 from os import urandom
 
 try:
     from locale import getpreferredencoding
-
     oem_encoding = getpreferredencoding()
 except Exception:
     oem_encoding = 'utf-8'
@@ -160,12 +160,12 @@ def unpack_windows_version(version_message):
     if len(version_message) != 8:
         raise ValueError('version field must be 8 bytes long')
 
-    if str == bytes:  # python 2
+    if str is bytes:  # Python 2
         return (unpack('<B', version_message[0])[0],
                 unpack('<B', version_message[1])[0],
                 unpack('<H', version_message[2:4])[0],
                 unpack('<B', version_message[7])[0])
-    else:  # python 3
+    else:  # Python 3
         return (int(version_message[0]),
                 int(version_message[1]),
                 int(unpack('<H', version_message[2:4])[0]),
@@ -488,5 +488,10 @@ class NtlmClient(object):
         return nt_challenge_response
 
     def ntowf_v2(self):
-        password_digest = hashlib.new('MD4', self._password.encode('utf-16-le')).digest()
+        passparts = self._password.split(':')
+        if len(passparts) == 2 and len(passparts[0]) == 32 and len(passparts[1]) == 32:
+            # The specified password is an LM:NTLM hash
+            password_digest = binascii.unhexlify(passparts[1])
+        else:
+            password_digest = hashlib.new('MD4', self._password.encode('utf-16-le')).digest()
         return hmac.new(password_digest, (self.user_name.upper() + self.user_domain).encode('utf-16-le')).digest()
