@@ -5,7 +5,7 @@
 #
 # Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2015, 2016, 2017 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -22,6 +22,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with ldap3 in the COPYING and COPYING.LESSER files.
 # If not, see <http://www.gnu.org/licenses/>.
+from binascii import hexlify
 
 from .. import STRING_TYPES
 
@@ -38,7 +39,13 @@ def to_stdout_encoding(value):
     if not isinstance(value, STRING_TYPES):
         value = str(value)
 
-    if str == bytes:  # python 2
-        return value.encode(repr_encoding, 'backslashreplace')
-    else:
-        return value.encode(repr_encoding, errors='backslashreplace').decode(repr_encoding, errors='backslashreplace')
+    if str is bytes:  # Python 2
+        try:
+            return value.encode(repr_encoding, 'backslashreplace')
+        except UnicodeDecodeError:  # Python 2.6
+            return hexlify(value)
+    else:  # Python 3
+        try:
+            return value.encode(repr_encoding, errors='backslashreplace').decode(repr_encoding, errors='backslashreplace')
+        except UnicodeDecodeError:
+            return hexlify(value)
