@@ -1,8 +1,11 @@
+"""
+"""
+
 # Created on 2013.05.15
 #
-# @author: Giovanni Cannata
+# Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -32,4 +35,4 @@ class Test(unittest.TestCase):
             rangeInt0ToMaxConstraint(-1)
         except Exception as e:
             msg = e.args[0]
-        self.assertEqual('ValueRangeConstraint(0, Integer(2147483647)) failed at: "-1"', msg)
+        self.assertTrue('ValueRangeConstraint(0, Integer(2147483647)) failed at' in msg)
