@@ -1,8 +1,11 @@
+"""
+"""
+
 # Created on 2013.06.06
 #
-# @author: Giovanni Cannata
+# Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -22,15 +25,16 @@
 
 import unittest
 
-from test import get_connection, drop_connection, \
-    add_user, random_id
+from test.config import get_connection, drop_connection, add_user, random_id
 
 
-testcase_id = random_id()
+testcase_id = ''
 
 
 class Test(unittest.TestCase):
     def setUp(self):
+        global testcase_id
+        testcase_id = random_id()
         self.connection = get_connection()
         self.delete_at_teardown = []
 
@@ -40,5 +44,8 @@ class Test(unittest.TestCase):
 
     def test_add(self):
         self.delete_at_teardown.append(add_user(self.connection, testcase_id, 'add-operation-1'))
+        self.assertEqual('success', self.delete_at_teardown[0][1]['description'])
 
+    def test_add_bytes(self):
+        self.delete_at_teardown.append(add_user(self.connection, testcase_id, 'add-operation-1', test_bytes=True))
         self.assertEqual('success', self.delete_at_teardown[0][1]['description'])
