@@ -1,8 +1,11 @@
+"""
+"""
+
 # Created on 2013.12.13
 #
-# @author: Giovanni Cannata
+# Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,14 +26,16 @@
 import unittest
 
 from ldap3 import Connection, LDIF, MODIFY_ADD, MODIFY_REPLACE, MODIFY_DELETE
-from test import test_base, generate_dn, test_name_attr, test_moved, random_id
+from test.config import test_base, generate_dn, test_name_attr, test_moved, random_id
 
 
-testcase_id = random_id()
+testcase_id = ''
 
 
 class Test(unittest.TestCase):
     def setUp(self):
+        global testcase_id
+        testcase_id = random_id()
         self.connection = Connection(server=None, client_strategy=LDIF)
         self.connection.open()
 
@@ -45,7 +50,7 @@ class Test(unittest.TestCase):
         if str != bytes:  # python3
             controls.append(('2.16.840.1.113719.1.27.103.7', False, bytearray('\u00e0\u00e0', encoding='UTF-8')))
         else:
-            controls.append(('2.16.840.1.113719.1.27.103.7', False, bytearray(unicode('\xe0\xe0', encoding='latin1'), encoding='UTF-8')))  # for python2 compatability
+            controls.append(('2.16.840.1.113719.1.27.103.7', False, bytearray(unicode('\xe0\xe0', encoding='latin1'), encoding='UTF-8')))  # for python2 compatibility
         controls.append(('2.16.840.1.113719.1.27.103.7', False, 'trailingspace '))
         self.connection.add(generate_dn(test_base, testcase_id, 'ldif-change-1'), 'inetOrgPerson', {'objectClass': 'inetOrgPerson', 'sn': 'ldif-change-1', test_name_attr: 'ldif-change-1'}, controls=controls)
         response = self.connection.response
@@ -63,73 +68,73 @@ class Test(unittest.TestCase):
     def test_delete_request_to_ldif(self):
         self.connection.strategy.order = dict(delRequest=['dn:', 'changetype', 'vers'])
         self.connection.delete(generate_dn(test_base, testcase_id, 'ldif-change-2'))
-        response = self.connection.response
-        self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-2,' + test_base in response)
-        self.assertTrue('changetype: delete' in response)
+        response = self.connection.response.replace('\r\n', '').replace(' ', '')
+        self.assertTrue('version:1' in response)
+        self.assertTrue('dn:' + test_name_attr + '=' + testcase_id + 'ldif-change-2,' + test_base in response)
+        self.assertTrue('changetype:delete' in response)
 
     def test_modify_dn_request_to_ldif(self):
         result = self.connection.modify_dn(generate_dn(test_base, testcase_id, 'ldif-change-3'), test_name_attr + '=' + testcase_id + 'ldif-change-4,' + test_base)
         if isinstance(result, int):
             self.connection.get_response(result)
-        response = self.connection.response
-        self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-3,' + test_base in response)
-        self.assertTrue('changetype: moddn' in response)
-        self.assertTrue('newrdn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-4,' + test_base in response)
-        self.assertTrue('deleteoldrdn: 1' in response)
+        response = self.connection.response.replace('\r\n', '').replace(' ', '')
+        self.assertTrue('version:1' in response)
+        self.assertTrue('dn:' + test_name_attr + '=' + testcase_id + 'ldif-change-3,' + test_base in response)
+        self.assertTrue('changetype:moddn' in response)
+        self.assertTrue('newrdn:' + test_name_attr + '=' + testcase_id + 'ldif-change-4,' + test_base in response)
+        self.assertTrue('deleteoldrdn:1' in response)
 
     def test_move_dn_request_to_ldif(self):
         result = self.connection.modify_dn(generate_dn(test_base, testcase_id, 'ldif-change-5'), test_name_attr + '=' + testcase_id + 'ldif-change-5', delete_old_dn=False, new_superior=test_moved)
         if isinstance(result, int):
             self.connection.get_response(result)
-        response = self.connection.response
-        self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-5,' + test_base in response)
-        self.assertTrue('changetype: modrdn' in response)
-        self.assertTrue('newrdn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-5' in response)
-        self.assertTrue('deleteoldrdn: 0' in response)
-        self.assertTrue('newsuperior: ' + test_moved in response)
+        response = self.connection.response.replace('\r\n', '').replace(' ', '')
+        self.assertTrue('version:1' in response)
+        self.assertTrue('dn:' + test_name_attr + '=' + testcase_id + 'ldif-change-5,' + test_base in response)
+        self.assertTrue('changetype:modrdn' in response)
+        self.assertTrue('newrdn:' + test_name_attr + '=' + testcase_id + 'ldif-change-5' in response)
+        self.assertTrue('deleteoldrdn:0' in response)
+        self.assertTrue('newsuperior:' + test_moved in response)
 
     def test_modify_add_to_ldif(self):
         result = self.connection.modify(generate_dn(test_base, testcase_id, 'ldif-change-6'), {'givenName': (MODIFY_ADD, ['givenname-6-modified'])})
         if isinstance(result, int):
             self.connection.get_response(result)
-        response = self.connection.response
-        self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-6,' + test_base in response)
-        self.assertTrue('changetype: modify' in response)
-        self.assertTrue('add: givenName' in response)
-        self.assertTrue('givenName: givenname-6-modified' in response)
+        response = self.connection.response.replace('\r\n', '').replace(' ', '')
+        self.assertTrue('version:1' in response)
+        self.assertTrue('dn:' + test_name_attr + '=' + testcase_id + 'ldif-change-6,' + test_base in response)
+        self.assertTrue('changetype:modify' in response)
+        self.assertTrue('add:givenName' in response)
+        self.assertTrue('givenName:givenname-6-modified' in response)
         self.assertEqual('-', response[-1])
 
     def test_modify_replace_to_ldif(self):
         result = self.connection.modify(generate_dn(test_base, testcase_id, 'ldif-change-7'), {'givenName': (MODIFY_REPLACE, ['givenname-7-replaced'])})
         if isinstance(result, int):
             self.connection.get_response(result)
-        response = self.connection.response
-        self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-7,' + test_base in response)
-        self.assertTrue('changetype: modify' in response)
-        self.assertTrue('replace: givenName' in response)
-        self.assertTrue('givenName: givenname-7-replaced' in response)
+        response = self.connection.response.replace('\r\n', '').replace(' ', '')
+        self.assertTrue('version:1' in response)
+        self.assertTrue('dn:' + test_name_attr + '=' + testcase_id + 'ldif-change-7,' + test_base in response)
+        self.assertTrue('changetype:modify' in response)
+        self.assertTrue('replace:givenName' in response)
+        self.assertTrue('givenName:givenname-7-replaced' in response)
         self.assertEqual('-', response[-1])
 
     def test_modify_delete_to_ldif(self):
         result = self.connection.modify(generate_dn(test_base, testcase_id, 'ldif-change-8'), {'givenName': (MODIFY_DELETE, ['givenname-8-deleted'])})
         if isinstance(result, int):
             self.connection.get_response(result)
-        response = self.connection.response
-        self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: ' + test_name_attr + '=' + testcase_id + 'ldif-change-8,' + test_base in response)
-        self.assertTrue('changetype: modify' in response)
-        self.assertTrue('delete: givenName' in response)
-        self.assertTrue('givenName: givenname-8-deleted' in response)
+        response = self.connection.response.replace('\r\n', '').replace(' ', '')
+        self.assertTrue('version:1' in response)
+        self.assertTrue('dn:' + test_name_attr + '=' + testcase_id + 'ldif-change-8,' + test_base in response)
+        self.assertTrue('changetype:modify' in response)
+        self.assertTrue('delete:givenName' in response)
+        self.assertTrue('givenName:givenname-8-deleted' in response)
         self.assertEqual('-', response[-1])
 
     def test_multiple_modify_to_ldif(self):
         # from rfc 2849 example
-        result = self.connection.modify('cn=Paula Jensen, ou=Product Development, dc=airius, dc=com',
+        result = self.connection.modify('cn=Paula Jensen,ou=Product Development,dc=airius,dc=com',
                                         {'postaladdress': (MODIFY_ADD, ['123 Anystreet $ Sunnyvale, CA $ 94086']),
                                          'description': (MODIFY_DELETE, []),
                                          'telephonenumber': (MODIFY_REPLACE, ['+1 408 555 1234', '+1 408 555 5678']),
@@ -138,7 +143,7 @@ class Test(unittest.TestCase):
             self.connection.get_response(result)
         response = self.connection.response
         self.assertTrue('version: 1' in response)
-        self.assertTrue('dn: cn=Paula Jensen, ou=Product Development, dc=airius, dc=com' in response)
+        self.assertTrue('dn: cn=Paula Jensen,ou=Product Development,dc=airius,dc=com' in response)
         self.assertTrue('changetype: modify' in response)
         self.assertTrue('delete: facsimiletelephonenumber' in response)
         self.assertTrue('facsimiletelephonenumber: +1 408 555 9876' in response)
