@@ -1,8 +1,11 @@
+"""
+"""
+
 # Created on 2013.06.06
 #
-# @author: Giovanni Cannata
+# Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,15 +26,16 @@
 import unittest
 
 from ldap3 import MODIFY_REPLACE, MODIFY_ADD, MODIFY_DELETE
-from test import random_id, get_connection, add_user, \
+from test.config import random_id, get_connection, add_user, \
     drop_connection
 
-
-testcase_id = random_id()
+testcase_id = ''
 
 
 class Test(unittest.TestCase):
     def setUp(self):
+        global testcase_id
+        testcase_id = random_id()
         self.connection = get_connection()
         self.delete_at_teardown = []
         self.delete_at_teardown.append(add_user(self.connection, testcase_id, 'modify-1', attributes={'givenName': 'givenname-1'}))
@@ -48,6 +52,15 @@ class Test(unittest.TestCase):
             result = self.connection.result
         self.assertEqual(result['description'], 'success')
 
+    def test_modify_replace_with_get_request(self):
+        result = self.connection.modify(self.delete_at_teardown[0][0], {'givenName': (MODIFY_REPLACE, ['givenname-1-replaced']), 'sn': (MODIFY_REPLACE, ['sn-replaced'])})
+        if not self.connection.strategy.sync:
+            _, result, request = self.connection.get_response(result, get_request=True)
+            self.assertEqual(request['type'], 'modifyRequest')
+        else:
+            result = self.connection.result
+        self.assertEqual(result['description'], 'success')
+
     def test_modify_add(self):
         result = self.connection.modify(self.delete_at_teardown[0][0], {'businessCategory': (MODIFY_ADD, ['businessCategory-2-added'])})
         if not self.connection.strategy.sync:
