@@ -1,8 +1,11 @@
+"""
+"""
+
 # Created on 2013.09.13
 #
-# @author: Giovanni Cannata
+# Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -24,7 +27,7 @@ import unittest
 
 from ldap3 import Server, OFFLINE_EDIR_8_8_8, SchemaInfo, DsaInfo
 from ldap3.protocol.rfc4512 import ObjectClassInfo, AttributeTypeInfo
-from test import test_server, get_connection, drop_connection
+from test.config import test_server, get_connection, drop_connection
 
 
 class Test(unittest.TestCase):
