@@ -1,8 +1,11 @@
+"""
+"""
+
 # Created on 2013.06.04
 #
-# @author: Giovanni Cannata
+# Author: Giovanni Cannata
 #
-# Copyright 2015 Giovanni Cannata
+# Copyright 2013, 2014, 2015, 2016 Giovanni Cannata
 #
 # This file is part of ldap3.
 #
@@ -23,23 +26,24 @@
 import unittest
 
 from ldap3.operation.search import parse_filter, MATCH_EQUAL, MATCH_EXTENSIBLE
+from test.config import test_auto_escape, test_auto_encode
 
 
 class Test(unittest.TestCase):
     def test_parse_search_filter_equality(self):
-        f = parse_filter('(cn=admin)', None)
+        f = parse_filter('(cn=admin)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EQUAL)
         self.assertEqual(f.elements[0].assertion['attr'], 'cn')
         self.assertEqual(f.elements[0].assertion['value'], b'admin')
 
     def test_parse_search_filter_equality_2(self):
-        f = parse_filter('(cn=a<=b=>c)', None)
+        f = parse_filter('(cn=a<=b=>c)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EQUAL)
         self.assertEqual(f.elements[0].assertion['attr'], 'cn')
         self.assertEqual(f.elements[0].assertion['value'], b'a<=b=>c')
 
     def test_parse_search_filter_extensible_syntax_1(self):
-        f = parse_filter('(cn:caseExactMatch:=Fred Flintstone)', None)
+        f = parse_filter('(cn:caseExactMatch:=Fred Flintstone)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EXTENSIBLE)
         self.assertEqual(f.elements[0].assertion['attr'], 'cn')
         self.assertEqual(f.elements[0].assertion['value'], b'Fred Flintstone')
@@ -47,7 +51,7 @@ class Test(unittest.TestCase):
         self.assertEqual(f.elements[0].assertion['dnAttributes'], None)
 
     def test_parse_search_filter_extensible_syntax_2(self):
-        f = parse_filter('(cn:=Betty Rubble)', None)
+        f = parse_filter('(cn:=Betty Rubble)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EXTENSIBLE)
         self.assertEqual(f.elements[0].assertion['attr'], 'cn')
         self.assertEqual(f.elements[0].assertion['value'], b'Betty Rubble')
@@ -55,7 +59,7 @@ class Test(unittest.TestCase):
         self.assertEqual(f.elements[0].assertion['dnAttributes'], None)
 
     def test_parse_search_filter_extensible_syntax_3(self):
-        f = parse_filter('(sn:dn:2.4.6.8.10:=Barney Rubble)', None)
+        f = parse_filter('(sn:dn:2.4.6.8.10:=Barney Rubble)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EXTENSIBLE)
         self.assertEqual(f.elements[0].assertion['attr'], 'sn')
         self.assertEqual(f.elements[0].assertion['value'], b'Barney Rubble')
@@ -63,7 +67,7 @@ class Test(unittest.TestCase):
         self.assertEqual(f.elements[0].assertion['dnAttributes'], True)
 
     def test_parse_search_filter_extensible_syntax_4(self):
-        f = parse_filter('(o:dn:=Ace Industry)', None)
+        f = parse_filter('(o:dn:=Ace Industry)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EXTENSIBLE)
         self.assertEqual(f.elements[0].assertion['attr'], 'o')
         self.assertEqual(f.elements[0].assertion['value'], b'Ace Industry')
@@ -71,7 +75,7 @@ class Test(unittest.TestCase):
         self.assertEqual(f.elements[0].assertion['dnAttributes'], True)
 
     def test_parse_search_filter_extensible_syntax_5(self):
-        f = parse_filter('(:1.2.3:=Wilma Flintstone)', None)
+        f = parse_filter('(:1.2.3:=Wilma Flintstone)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EXTENSIBLE)
         self.assertEqual(f.elements[0].assertion['attr'], None)
         self.assertEqual(f.elements[0].assertion['value'], b'Wilma Flintstone')
@@ -79,7 +83,7 @@ class Test(unittest.TestCase):
         self.assertEqual(f.elements[0].assertion['dnAttributes'], None)
 
     def test_parse_search_filter_extensible_syntax_6(self):
-        f = parse_filter('(:DN:2.4.6.8.10:=Dino)', None)
+        f = parse_filter('(:DN:2.4.6.8.10:=Dino)', None, test_auto_escape, test_auto_encode)
         self.assertEqual(f.elements[0].tag, MATCH_EXTENSIBLE)
         self.assertEqual(f.elements[0].assertion['attr'], None)
         self.assertEqual(f.elements[0].assertion['value'], b'Dino')
